#include "main.h"
#include "led.h"
//------------------------------------------------
#define DEV_ADDR 0xAE
#define address 0x0A
//------------------------------------------------
unsigned int TIM1_Count=0;
const unsigned char at24c_data_write[]={0x14,0x13,0x12,0x11,0x10,
0x0F,0x0E,0x0D,0x0C,0x0B,
0x0A,0x09,0x08,0x07,0x06,
0x05,0x04,0x03,0x02,0x01};
unsigned char at24c_date[20];
//------------------------------------------------
void interrupt timer0()
{
  TIM0_Callback();
  T0IF=0;
}
//------------------------------------------------
void I2C_Idle()
{
  while(READ_WRITE);
  ZERO=0;
  while(ZERO)
  {
    SSPCON2&0x1f;
  }
}
//------------------------------------------------
void I2C_IntWait(void)
{
  while(!SSPIF);
  SSPIF=0;
}
//------------------------------------------------
void I2C_SendByte(unsigned char data)
{
  SSPBUF = data;
  I2C_IntWait();
}
//------------------------------------------------
unsigned char I2C_ReceiveByte_Ack(void)
{
  unsigned char data = 0;
  RCEN=1;
  while(!SSPIF);
  data=SSPBUF;
  I2C_IntWait();
  ACKDT=0;
  ACKEN=1;
  I2C_IntWait();
  return data;
}
//------------------------------------------------
unsigned char I2C_ReceiveByte_Nack(void)
{
  unsigned char data = 0;
  RCEN=1;
  while(!SSPIF);
  data=SSPBUF;
  while(!SSPIF);
  SSPIF=0;
  ACKDT=1;
  ACKEN=1;
  while(!SSPIF);
  SSPIF=0;
  return data;
}
//------------------------------------------------
void I2C_StartCondition(void)
{
  I2C_Idle();
  SEN=1;
  I2C_IntWait();
}
//------------------------------------------------
void I2C_StopCondition(void)
{
  PEN=1;
  NOP();
}
//------------------------------------------------
void AT24C_WriteBytes (unsigned int addr,unsigned char *buf, unsigned int bytes_count)
{
  unsigned int i;
  I2C_SendByte(DEV_ADDR);
  I2C_SendByte(addr<<8);
  I2C_SendByte((unsigned char)addr);
  while(BF);
  for(i=0;i<bytes_count;i++)
  {
    I2C_SendByte(buf[i]);
  }
}
//------------------------------------------------
void AT24C_ReadBytes (unsigned int addr,unsigned char *buf, unsigned int bytes_count)
{
  unsigned int i;
  I2C_SendByte(DEV_ADDR);
  I2C_SendByte(addr<<8);
  I2C_SendByte((unsigned char)addr);
  while(BF);
  I2C_Idle();
  SSPIF=0;
  RSEN=1;
  I2C_IntWait();
  I2C_SendByte(DEV_ADDR|0x01);
  for(i=0;i<bytes_count;i++)
  {
    if(i<(bytes_count-1))
    {
      buf[i] = I2C_ReceiveByte_Ack();
    }
    else
    {
      buf[i] = I2C_ReceiveByte_Nack();
    }
  }
}
//------------------------------------------------
void main()
{
  int i;
  TRISB = 0x00;
  PORTB = 0xFF;
  TRISA = 0x00;
  PORTA = 0x00;
  OPTION_REG=0b00000010; //Prescaler
  INTCON=0xA0;
  TMR0=0;
  TRISC=0x18;//RC3 and RC4 Input (SDC and SDA)
  // SSPM3:SSPM0 = 1000 (I2C Master mode, clock = FOSC/(4 * (SSPADD + 1))
  // = 4000000/(4 * (9 + 1)) = 100000 Hz)
  // SKE=1(Disable SMBus specific inputs)
  // SKP=1(Release clock)
  // SMP=1 (Slew rate control disabled for standard speed mode (100 kHz and 1 MHz))
  // SSPEN= 1 (Enables the serial port and configures the SDA and SCL pins as the serial port pins)
  SSPADD=0X09;
  SSPSTAT=0X80;
  SSPCON=0X38;
  //write
  /*
  I2C_StartCondition();
  AT24C_WriteBytes(0x000A,at24c_data_write,20);
  I2C_StopCondition();
  __delay_ms(5);
  */
  //read
  SSPIF=0;
  I2C_StartCondition();
  AT24C_ReadBytes(0x000A,at24c_date,20);
  I2C_StopCondition();
  SSPIF=0;
  __delay_ms(1);
  for(i=0;i<20;i++)
  {
    ledprint(at24c_date[i]);
    __delay_ms(500);
  }
  while(1)
  {
    __delay_ms(5);
  }
}
//------------------------------------------------
